#include <stdlib.h>
#include <ctype.h>
#include <tcl.h>
#include <stdio.h>
#include <math.h>
#include <string.h>		/* strcat */

#include "rpm-internal.h"

static char rcsid[] = "$Id: rpmElementary.c,v 1.4 1999/08/09 13:51:11 martin Exp $";

#ifdef USE_FMOD
static double remainder(double a, double b)
{
	return fmod(a, b);
}
#endif

/*
**	v1 v2 mod
*/
int rpmModSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int i, n, n2;
    rpmBinary a, b, c;

    b = rpmPop();
    a = rpmPop();
    n = rpmLength(a);
    n2 = rpmLength(b);
    c = rpmNewBinaryLike(a);
    for(i = 0; i < n; i++) {
      if(i < n2)
	c.v[i] = fmod(a.v[i], b.v[i]);
      else
	c.v[i] = fmod(a.v[i], b.v[n2 - 1]);
      }
    rpmFreeBinary(a);
    rpmFreeBinary(b);
    rpmPushB(c);
    return TCL_OK;
}

/*
**	v1 v2 remainder
*/
int rpmRemainderSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int i, n, n2;
    rpmBinary a, b, c;

    b = rpmPop();
    a = rpmPop();
    n = rpmLength(a);
    n2 = rpmLength(b);
    c = rpmNewBinaryLike(a);
    for(i = 0; i < n; i++) {
      if(i < n2)
	c.v[i] = remainder(a.v[i], b.v[i]);
      else
	c.v[i] = remainder(a.v[i], b.v[n2 - 1]);
      }
    rpmFreeBinary(a);
    rpmFreeBinary(b);
    rpmPushB(c);
    return TCL_OK;
}

/*
**	v1 v2 atan2
*/
int rpmAtan2SubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int i, n, n2;
    rpmBinary a, b, c;

    b = rpmPop();
    a = rpmPop();
    n = rpmLength(a);
    n2 = rpmLength(b);
    c = rpmNewBinaryLike(a);
    for(i = 0; i < n; i++) {
      if(i < n2)
	c.v[i] = atan2(a.v[i], b.v[i]);
      else
	c.v[i] = atan2(a.v[i], b.v[n2 - 1]);
      }
    rpmFreeBinary(a);
    rpmFreeBinary(b);
    rpmPushB(c);
    return TCL_OK;
}

int applySimpleF(Tcl_Interp* interp, double (*f)(double))
{
    int i, n;
    rpmBinary a, b;

    a = rpmPop();
    n = rpmLength(a);
    b = rpmNewBinaryLike(a);
    for(i = 0; i < n; i++)
	b.v[i] = f(a.v[i]);
    rpmFreeBinary(a);
    rpmPushB(b);
    return TCL_OK;
}
/*
**	floor
*/
int rpmFloorSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, floor);
}
/*
**	ceil
*/
int rpmCeilSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, ceil);
}
/*
**	abs
*/
int rpmAbsSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, fabs);
}
/*
**	rint
*/
int rpmRintSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, rint);
}
/*
**	exp
*/
int rpmExpSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, exp);
}
/*
**	log
*/
int rpmLogSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, log);
}
/*
**	log10
*/
int rpmLog10SubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, log10);
}
/*
**	sqrt
*/
int rpmSqrtSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, sqrt);
}
/*
**	sin
*/
int rpmSinSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, sin);
}
/*
**	cos
*/
int rpmCosSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, cos);
}
/*
**	tan
*/
int rpmTanSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, tan);
}
/*
**	acos
*/
int rpmAcosSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, acos);
}
/*
**	asin
*/
int rpmAsinSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, asin);
}
/*
**	atan
*/
int rpmAtanSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applySimpleF(interp, atan);
}


