#include <stdlib.h>
#include <ctype.h>
#include <tcl.h>
#include <stdio.h>
#include <math.h>
#include <string.h>		/* strcat */

#include "rpm-internal.h"

static char rcsid[] = "$Id: rpmMisc.c,v 1.9 1999/08/09 13:51:11 martin Exp $";

/*
**	min max list scaleto
*/
int rpmScaletoSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    double omin, omax;
    int n;
    double minv, maxv;
    double a, b;
    int i;

    rlist = rpmPop();
    omax = rpmPopScalar();
    omin = rpmPopScalar();
    n = rpmLength(rlist);

    if(n < 1) {
	Tcl_SetResult(interp, "can't scale zero-length sequence",
		      TCL_STATIC);
	return TCL_ERROR;
    }
    minv = maxv = rlist.v[0];
    for(i = 1; i < n; i++) {
	if(rlist.v[i] > maxv)
	    maxv = rlist.v[i];
	if(rlist.v[i] < minv)
	    minv = rlist.v[i];
    }

    if(minv >= maxv)
	b = 0.0;
    else
	b = (omax - omin) / (maxv - minv);
    a = omax - b * maxv;
    retv = rpmNewBinaryLike(rlist);
    for(i = 0; i < n; i++)
	retv.v[i] = a + b * rlist.v[i];
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**   list first last range
*/
int rpmRangeSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    int first, last;
    int n;
    int i;

    last = rpmPopScalar();
    first = rpmPopScalar();
    rlist = rpmPop();
    n = rpmLength(rlist);
    if(last >= n)
      last = n - 1;
    if(last < 0)
      last = 0;
    if(first > last)
      first = last;
    if(first < 0)
      first = 0;
    retv = rpmNewBinaryD1(last - first + 1);
    for(i = 0; i < (last - first + 1); i++)
      retv.v[i] = rlist.v[i + first];
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

static int rpmComp(const void* a, const void* b)
{
  double d =  *((rpmFloat *) a) - * ((rpmFloat *) b);
  if(d < 0.0)
    return -1;
  if(d > 0.0)
    return 1;
  return 0;
}

/*
**    list sort
*/
int rpmSortSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    int n;

    rlist = rpmPop();
    n = rpmLength(rlist);

    if(n < 1) {
	Tcl_SetResult(interp, "can't sort zero-length sequence",
		      TCL_STATIC);
	return TCL_ERROR;
    }
    retv = rpmNewBinaryD1(n);
    memmove(retv.v, rlist.v, n * sizeof(retv.v[0]));
    qsort(retv.v, n, sizeof(retv.v[0]), rpmComp);
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	list length
*/
int rpmLengthSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;

    rlist = rpmPop();
    retv = rpmNewBinaryD1(1);
    retv.v[0] = rpmLength(rlist);
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	list max
*/
int rpmMaxSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    double mval;
    int i;
    int n;

    rlist = rpmPop();
    n = rpmLength(rlist);
    mval = rlist.v[0];
    for(i = 1; i < n; i++)
	if(rlist.v[i] > mval)
	    mval = rlist.v[i];

    retv = rpmNewBinaryD1(1);
    retv.v[0] = mval;
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	list min
*/
int rpmMinSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    double mval;
    int i;
    int n;

    rlist = rpmPop();
    n = rpmLength(rlist);
    mval = rlist.v[0];
    for(i = 1; i < n; i++)
	if(rlist.v[i] < mval)
	    mval = rlist.v[i];

    retv = rpmNewBinaryD1(1);
    retv.v[0] = mval;
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	list sum
*/
int rpmSumSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    double mval;
    int i;
    int n;

    rlist = rpmPop();
    n = rpmLength(rlist);
    mval = 0.0;
    for(i = 0; i < n; i++)
	mval += rlist.v[i];

    retv = rpmNewBinaryD1(1);
    retv.v[0] = mval;
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	list exponent psum
*/
int rpmPsumSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    double mval, a;
    int i;
    int n;

    a = rpmPopScalar();
    rlist = rpmPop();
    n = rpmLength(rlist);
    mval = 0.0;
    for(i = 0; i < n; i++)
	mval += pow(rlist.v[i], a);

    retv = rpmNewBinaryD1(1);
    retv.v[0] = mval;
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	list level threshold
*/
int rpmThresholdSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    double a;
    int i;
    int n;

    a = rpmPopScalar();
    rlist = rpmPop();
    n = rpmLength(rlist);
    for(i = 0; i < n; i++)
      if(rlist.v[i] >= a)
	break;

    retv = rpmNewBinaryD1(1);
    retv.v[0] = (i < n) ? i : -1;
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	list sumsq
*/
int rpmSumsqSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    double mval;
    int i;
    int n;

    rlist = rpmPop();
    n = rpmLength(rlist);
    mval = 0.0;
    for(i = 0; i < n; i++)
	mval += rlist.v[i] * rlist.v[i];

    retv = rpmNewBinaryD1(1);
    retv.v[0] = mval;
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	stats list
**	returns {
**          max index-of-max min index-of-min <y> <y*y> <y*y*y>
**	}
*/
int rpmStatsSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int i;
    int imin, imax;
    rpmBinary a, r;
    int n;

    a = rpmPop();
    n = rpmLength(a);
    r = rpmNewBinaryD1(7);
    r.v[0] = a.v[0];
    imin = 0;
    r.v[2] = a.v[0];
    imax = 0;
    r.v[4] = a.v[0];
    r.v[5] = a.v[0] * a.v[0];
    r.v[6] = a.v[0] * a.v[0] *a.v[0];

    for(i = 1; i < n; i++) {
	if(a.v[i] > r.v[0]) {
	    r.v[0] = a.v[i];
	    imax = i;
	}
	if(a.v[i] < r.v[2]) {
	    r.v[2] = a.v[i];
	    imin = i;
	}
	r.v[4] += a.v[i];
	r.v[5] += a.v[i] * a.v[i];
	r.v[6] += a.v[i] * a.v[i] * a.v[i];
    }
    r.v[1] = imax;
    r.v[3] = imin;
    r.v[4] /= n;
    r.v[5] /= n;
    r.v[6] /= n;
    rpmFreeBinary(a);
    rpmPushB(r);
    return TCL_OK;
}

/*
**	from to list abscissate
*/
int rpmAbscissateSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int k, i;
    double xmin, xmax, dx;
    double x;
    int npts;

    rpmBinary lin, lout;

    lin = rpmPop();
    xmax = rpmPopScalar();
    xmin = rpmPopScalar();
    npts = rpmLength(lin);
    if(npts > 1)
	dx = (xmax - xmin)/(npts - 1.0);
    else
	dx = 0.0;
    lout = rpmNewBinaryD1(2 * npts);
    k = 0;
    x = xmin;
    for(i = 0; i < npts; i++) {
	lout.v[k++] = x;
	lout.v[k++] = lin.v[i];
       x += dx;
    }
    rpmFreeBinary(lin);
    rpmPushB(lout);
    return TCL_OK;
}


/*
**	list count decimate
*/
int rpmDecimateSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int decim;
    int i;
    int nin;
    int nout;
    rpmBinary lin, lout;

    decim = rpmPopScalar();
    lin = rpmPop();
    nin = rpmLength(lin);
    if(decim <= 1)
	decim = 1;
    nout = 1 + ((nin - 1) / decim);
    lout = rpmNewBinaryD1(nout);
    for(i = 0; i < nout; i++)
	lout.v[i] = lin.v[i * decim];
    rpmFreeBinary(lin);
    rpmPushB(lout);
    return TCL_OK;
}


/*
**	toplot nine-scalar-arguments list
*/
int rpmToplotSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    double pleft, pright, pmin, pmax;
    double tfirst, tlast;
    int minpoints;
    double tzero, dt;

    double ffirst, flast;
    double oldffirst, oldflast;
    int newminpoints;
    int dec;
    double tmp;

    int ifirst, ilast, idelta, nsteps;
    double vmin, vmax;
    double v;
    double ay, by;
    double ax, bx;
    int i;
    int k;
    double x, y;

    rpmBinary lin, lout;
    int nvals;

    lin = rpmPop();
    nvals = rpmLength(lin);
    dt = rpmPopScalar();
    tzero = rpmPopScalar();
    minpoints = rpmPopScalar();
    tlast = rpmPopScalar();
    tfirst = rpmPopScalar();
    pmax = rpmPopScalar();
    pmin = rpmPopScalar();
    pright = rpmPopScalar();
    pleft = rpmPopScalar();

/*
**	compute sample indices corresponding to tfirst and tlast;
**	these are f.p. indices.
*/
    ffirst = (tfirst - tzero)/dt;
    flast = (tlast - tzero)/dt;
    oldffirst = ffirst;
    oldflast = flast;
/*
**	if ffirst is less than zero, adjust tfirst, ffirst, and pleft
**	accordingly.
*/
    if (ffirst < 0.0) {
	pleft = pleft + (pright - pleft) * (0.0 - ffirst)/(flast - ffirst);
	ffirst = 0.0;
	tfirst = tzero;
    }
/*
**	do something similar to tlast, flast, and pright if flast
**	exceeds the number of points in the series.
*/
    if (flast > (nvals - 1)) {
	pright = pleft + (pright - pleft) * ((nvals - 1) - ffirst)
	    / (flast - ffirst);
	flast = nvals - 1;
	tlast = tzero + flast * dt;
    }
    bx = (pright - pleft) / (flast - ffirst);
    ax = pright - bx * flast;
/*
**	if decimation is allowed, compute dec now; otherwise
**	make it unity.
*/
    newminpoints = minpoints * (flast - ffirst) / (oldflast - oldffirst);
    if(newminpoints < minpoints) ++newminpoints;
    if(minpoints > 0) {
	tmp = (int)(flast) - (int)(ffirst + 0.99999) + 1;
	dec = tmp / (1.0 * newminpoints);
	if (dec < 1)
	    dec = 1;
    } else {
	dec = 1;
    }
/*
**	finalize indices for extracting y's
*/
    ifirst = ffirst + 0.9999999;
    idelta = dec;
    nsteps = (((int) flast) - ifirst)/idelta;
    ilast  = ifirst + nsteps * idelta;
/*
**	compute min and max values over plotting range and
**	the mapping parameters for y.
*/
    vmin = vmax = lin.v[ifirst];
    for(i = ifirst; i <= ilast; i += idelta) {
	v = lin.v[i];
	if(v < vmin) vmin = v;
	if(v > vmax) vmax = v;
    }
    by = (pmax - pmin) / (vmax - vmin);
    ay = pmax - by * vmax;
/*
**	make space for the answer
*/
    lout = rpmNewBinaryD1(2 * nsteps);
/*
**	make it
*/
    k = 0;
    for(i = ifirst; i <= ilast; i += idelta) {
	x = ax + bx * i;
	lout.v[k++] = x;
	y = ay + by * lin.v[i];
	lout.v[k++] = y;
    }
    rpmFreeBinary(lin);
    rpmPushB(lout);
    return TCL_OK;
}
